/*
 * This file is a component of thundr, a software library from 3wks.
 * Read more: http://www.3wks.com.au/thundr
 * Copyright (C) 2013 3wks, <thundr@3wks.com.au>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.threewks.thundr.googleapis.bigquery;

import java.io.IOException;
import java.util.List;

import com.google.api.services.bigquery.Bigquery;

/**
 * Service for interacting with Google BigQuery
 * 
 * https://developers.google.com/bigquery/
 * https://cloud.google.com/products/big-query
 */
public interface GoogleBigQueryService {

	/**
	 * Return the underlying {@link Bigquery} object allowing for low level API calls that are not supported in this class.
	 * 
	 * @return the {@link Bigquery} object.
	 */
	public Bigquery getBigQuery();

	/**
	 * Execute a query asynchronously.
	 * 
	 * @param query the query to run.
	 * @param projectId the id of the project the query should be run in.
	 * @return the id of the query job.
	 * @throws IOException if there is an error running the query.
	 */
	public String queryAsync(String query, String projectId) throws IOException;

	/**
	 * Get the results of an asynchronous query.
	 * 
	 * @param jobId the id of a job for a previously run query.
	 * @param projectId the id of the project the query was run in.
	 * @param includeColumnNames whether to include the column names as the first row of the results.
	 * @return a list containing an entry for each row and an optional header row, or null if the results of the query are not ready yet.
	 * @throws IOException if there is an getting the results of the query.
	 */
	public List<List<String>> getAsyncQueryResults(String jobId, String projectId, boolean includeColumnNames) throws IOException;

	/**
	 * Load CSV data into a table. Note that this method assumes the CSV data is small enough to be buffered in memory.
	 * 
	 * @param projectId the id of the project to load the data into.
	 * @param datasetId the id of the dataset to load the data into.
	 * @param tableId the id of the table to load the data into.
	 * @param csvData the CSV data to load into the table.
	 * @return the id of the load job.
	 * @throws IOException if there is an error loading the data.
	 */
	public String executeLoad(String projectId, String datasetId, String tableId, String csvData) throws IOException;

	/**
	 * Get the status of a job.
	 * 
	 * @param jobId the id of the job.
	 * @param projectId the id of the project.
	 * @return the {@link BigQueryJobStatus}.
	 */
	public BigQueryJobStatus getJobStatus(String jobId, String projectId);

	/**
	 * Load CSV data into a table from Google Cloud Storage.
	 * 
	 * @param projectId the id of the project to load the data into.
	 * @param datasetId the id of the dataset to load the data into.
	 * @param tableId the id of the table to load the data into.
	 * @param sourceUris the URIs of the source data in Google Cloud Storage.
	 * @return the id of the load job.
	 * @throws IOException if there is an error loading the data.
	 */
	public String executeLoadFromCloudStorage(String projectId, String datasetId, String tableId, List<String> sourceUris) throws IOException;

	/**
	 * Create a new table and load CSV data into it from Google Cloud Storage.
	 * 
	 * @param projectId the id of the project to load the data into.
	 * @param datasetId the id of the dataset to load the data into.
	 * @param tableId the id of the table to create.
	 * @param schema the table schema.
	 * @param sourceUris the URIs of the source data in Google Cloud Storage.
	 * @return the id of the load job.
	 * @throws IOException if there is an error loading the data.
	 */
	String createTableAndLoadFromGoogleCloudStorage(String projectId, String datasetId, String tableId, String schema, List<String> sourceUris) throws IOException;

}
